/*============================================================================*/
/*                        L a b W i n d o w s / C V I                         */
/*----------------------------------------------------------------------------*/
/*    Copyright (c) National Instruments 1987-1996.  All Rights Reserved.     */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/* Title:       daqctrl.h                                                     */
/* Purpose:     Provides support for DAQ Controls                             */
/*                                                                            */
/*============================================================================*/


#ifndef DAQ_CTRL_HEADER
#define DAQ_CTRL_HEADER

#include <userint.h>
#include "easyio.h"
#include "custsupp.h"

#ifdef __cplusplus
    extern "C" {
#endif

enum {
    ATTR_DAQ_CTRL_DEVICE                        = 3000,
    ATTR_DAQ_CTRL_CHANNEL_STRING,
    ATTR_DAQ_CTRL_CHANNEL_STRING_LENGTH,
    ATTR_DAQ_CTRL_CHANNEL_STRING_COPY,
    ATTR_DAQ_CTRL_HIGH_LIMIT,
    ATTR_DAQ_CTRL_LOW_LIMIT,
    ATTR_DAQ_CTRL_REFRESH_FREQUENCY,
    ATTR_DAQ_CTRL_ENABLED,
    ATTR_DAQ_CTRL_IO_DIRECTION,
    ATTR_DAQ_CTRL_VISIBLE,
    ATTR_DAQ_CTRL_ALLOW_RCLICK_EDIT,
    ATTR_DAQ_CTRL_EDIT_DIALOG_TITLE,
    ATTR_DAQ_CTRL_EDIT_DIALOG_TITLE_LENGTH,
    ATTR_DAQ_CTRL_EDIT_DIALOG_TITLE_COPY,
    ATTR_DAQ_CTRL_EDIT_DIALOG_DISPLAYED,
    ATTR_DAQ_CTRL_EDIT_DIALOG_MODAL,
    ATTR_DAQ_CTRL_TIE_CTRL_RANGE_TO_LIMITS,
    ATTR_DAQ_CTRL_VERSION,
    ATTR_DAQ_CTRL_ERROR_NOTIFICATION
    };


#define ATTR_DAQ_CTRL_DEVICE_DEFAULT                    1   /* if no device is found */
#define ATTR_DAQ_CTRL_CHANNEL_STRING_DEFAULT            "0"
#define ATTR_DAQ_CTRL_HIGH_LIMIT_DEFAULT                4.999
#define ATTR_DAQ_CTRL_LOW_LIMIT_DEFAULT                 -4.999
#define ATTR_DAQ_CTRL_ENABLED_DEFAULT                   TRUE
#define ATTR_DAQ_CTRL_VISIBLE_DEFAULT                   TRUE
#define ATTR_DAQ_CTRL_ALLOW_RCLICK_EDIT_DEFAULT         TRUE
#define ATTR_DAQ_CTRL_EDIT_DIALOG_DISPLAYED_DEFAULT     FALSE
#define ATTR_DAQ_CTRL_EDIT_DIALOG_MODAL_DEFAULT         FALSE
#define ATTR_DAQ_CTRL_TIE_CTRL_RANGE_TO_LIMITS_DEFAULT  FALSE
#define ATTR_DAQ_CTRL_ERROR_NOTIFICATION_DEFAULT        ERR_NOTIFY_BEEP_AND_BOX

typedef enum
    {
    DONT_REVERT,
    SOME_CHANGES_WILL_REVERT,
    ALL_CHANGES_WILL_REVERT
    } RevertReply;

#define MIN_DEVICE 1
#define MAX_DEVICE 16

#define TIE_CTRL_RANGE_TO_LIMITS_MAX                        10000.0
#define TIE_CTRL_RANGE_TO_LIMITS_MIN                        -10000.0

#define MAX_EDIT_DIALOG_TITLE_LEN       255
#define MAX_CHANNEL_STRING_LEN          255

#define DEVICE_TAG                      "Device"
#define BETA_DEVICE_TAG                 "Board"
#define CHANNEL_TAG                     "Channel String"
#define HIGH_LIMIT_TAG                  "High Limit"
#define LOW_LIMIT_TAG                   "Low Limit"
#define ENABLED_TAG                     "Enabled"
#define IO_DIRECTION_TAG                "I/O Direction"
#define VISIBLE_TAG                     "Visible"
#define ALLOW_RIGHT_CLICK_EDIT_TAG      "Allow Right Click Editing"
#define DIALOG_TITLE_TAG                "Editing Dialog Title"
#define DIALOG_MODAL_TAG                "Editing Dialog Modal"
#define TIE_TO_LIMITS_TAG               "Tie Control Range To Input Limits"
#define VERSION_TAG                     "Version"
#define ERROR_NOTIFICATION_TAG          "Error Notification"

#define MIN_DAQ_ERROR_DISPLAY_TIME_TILL_CLEARED     1.5
#define MIN_CTRL_ERROR_DISPLAY_TIME_TILL_CLEARED    10.0
#define MIN_ERROR_DISPLAY_TIME                      1.5

#define DAQ_CTRL_MIN_ACTUAL_FREQ_UPDATE_INTERVAL    1.0

#define DAQ_CTRL_HELP_FILE              "daqecho.hlp"

    /*  values for ATTR_DAQ_CTRL_ERROR_NOTIFICATION */
#define ERR_NOTIFY_BEEP_AND_BOX             0
#define ERR_NOTIFY_BEEP_ONLY                1
#define ERR_NOTIFY_BOX_ONLY                 2
#define ERR_NOTIFY_NONE                     3

typedef struct DaqCtrlStructTag *DaqCtrl;

typedef int     (CVIFUNC * GetAttrFunc)(int panel, int ctrl, int attribute, void *value);
typedef int     (CVIFUNC_C * SetAttrFunc)(int panel, int ctrl, int attribute, ...);
typedef int     (CVIFUNC * BuildDialogFunc)(DaqCtrl daqCtrl);
typedef int     (CVIFUNC * SaveSettingsFunc)(DaqCtrl daqCtrl, char *settingsName);
typedef int     (CVIFUNC * RecallSettingsFunc)(DaqCtrl daqCtrl, char *settingsName, char *errorDescription);
typedef int     (CVIFUNC * UpdateDialogDimmingFunc)(DaqCtrl daqCtrl);


typedef int     (CVIFUNC * RevertAttributesFunc)(DaqCtrl daqCtrl);
typedef int     (CVIFUNC * ApplyDialogSettingsFunc)(DaqCtrl daqCtrl);
typedef int     (CVIFUNC * GetErrorStringFunc)(DaqCtrl daqCtrl, int errorCode, char **errorTitle, char **errorMessage);
typedef int     (CVIFUNC * ErrorNotifyFunc)(DaqCtrl daqCtrl, int errorCode, char *errorMsg);
typedef int     (CVIFUNC * ConfirmRevertFunc)(DaqCtrl daqCtrl, RevertReply *reply);
typedef int     (CVIFUNC * RememberAttributesFunc)(DaqCtrl daqCtrl);
typedef int     (CVIFUNC * CopyAttributesToDialogFunc)(DaqCtrl daqCtrl);


    /* Macro containing the operation fields common to all daq controls */
#define COMMON_DAQ_CTRL_OPERATION_FIELDS            \
    int                         panel;              \
    int                         baseCtrl;           \
                                                    \
        /* functions supplied by derived classes */ \
    GetAttrFunc                 getAttribute;       \
    SetAttrFunc                 setAttribute;       \
    BuildDialogFunc             buildDialog;        \
    SaveSettingsFunc            saveSettings;       \
    RecallSettingsFunc          recallSettings;     \
    UpdateDialogDimmingFunc     updateDialogDimming;\
    GetErrorStringFunc          getErrorString;     /* optional */ \
    ErrorNotifyFunc             errorNotifyFunc;    /* optional */ \
    ConfirmRevertFunc           confirmRevertFunc;  /* optional */ \
    ApplyDialogSettingsFunc     applyDialogSettings;\
    RevertAttributesFunc        revertAttributes;\
    RememberAttributesFunc      rememberAttributes; \
    CopyAttributesToDialogFunc  copyAttributesToDialog;\
    int                         device;         \
    char                        *channelString; \
    double                      highLimit;      \
    double                      lowLimit;       \
    int                         enabled;        \
    int                         ioDirection;    \
    int                         allowRightClickEdit;    \
    int                         tieCtrlRangeToLimits;   \
    char                        settingsName[SETTING_NAME_BUF_SIZE];    \
    char                        *settingsPrefix;        \
    int                         version;    \
    int                         lastRecalledVersion; \
    int                         recallingSettings;  /* must be set and cleared by derived class */ \
    int                         revertingSettings;  /* must be set and cleared by derived class */ \
    int                         channelStringIsRevertable; \
    int                         errorBoxDecoration; \
    int                         errorNotification;  \
    int                         autoErrorClearingNeeded;    \


    /* Macro containing the dialog related fields common to all daq controls */
#define COMMOM_DAG_CTRL_DLG_FIELDS  \
    int                     dialog;             \
    int                     dialogDisplayed;    \
    int                     isDisplayedModally; \
    int                     shouldBeDisplayedModally;   \
    double                  savedHighLimit;             \
    double                  savedLowLimit;              \
    int                     savedDevice;                \
    char                    savedChannelString[MAX_CHANNEL_STRING_LEN+1];   \
    char                    savedTitleString[256];      \
    int                     savedEnabled;               \
    int                     savedIODirection;           \
    int                     savedAllowRightClickEdit;   \
    int                     savedModal;                 \
    int                     savedVisible;               \
    int                     savedTieLimits;             \
    char                    savedSettingsName[SETTING_NAME_BUF_SIZE];   \
    int                     savedErrorNotification;     \
    int                     changed;                    \
    int                     deviceNumeric;              \
    int                     channelStringCtrl;          \
    int                     highLimitNumeric;           \
    int                     lowLimitNumeric;            \
    int                     errorNotificationRing;      \
    int                     allowRightClickEditCheckBox;    \
    int                     visibleCheckBox;            \
    int                     tieLimitsCheckBox;          \
    int                     helpButton;                 \
    int                     modalCheckBox;              \
    int                     ioDirectionRing;            \
    int                     enabledCheckBox;            \
    int                     titleString;                \
    int                     okButton;                   \
    int                     cancelButton;               \
    int                     configFileString;           \
    int                     configNameString;           \
    int                     saveButton;                 \
    int                     recallButton;               \
    int                     deleteConfigButton;         \
    int                     sheetGroupCtrl;             \
    int                     errorTextMsg;               \
    int                     errorTextBox;               \
    int                     heightWithoutErrorExplanation;  \
    int                     heightWithErrorExplanation; \
    int                     updatingCtrlFromDialog;     \
    int                     closeTheDialog;     /* set to dismiss the edit popup when in DaqCtrl_EditPopup() */ \
    ListType                ctrlsWithLabelsOnLeft;      \
    ListType                daqTabCtrls;                \
    ListType                configTabCtrls;             \
    ListType                generalTabCtrls;            \
    ListType                generalTabCtrlsToDistribute;    \
    ListType                ctrlsOnLeft;                    \
    ListType                ctrlListsToDistribute;          \
    int                     errorDisplayVisible;            \
    int                     currentErrorCode;               \
    int                     displayedErrorCode;             \
    double                  errorClearableTime;             \
    double                  errorReplaceableTime;           \
    int                     vMargin;                        \
    int                     hMargin;                        \
    int                     minDialogWidth;                 \
    char                    *ctrlTypeName;                  \
    int                     dialogEvent; /* the event the editing dialog is currently processing, or 0 if none */ \
    char                    dialogTitle[MAX_EDIT_DIALOG_TITLE_LEN + 1]; \


    /* Macro containing the all the fields common to all daq controls */
#define COMMON_DAQ_CTRL_FIELDS  \
    COMMON_DAQ_CTRL_OPERATION_FIELDS    \
    COMMOM_DAG_CTRL_DLG_FIELDS      \

typedef struct DaqCtrlStructTag
    {
    COMMON_DAQ_CTRL_FIELDS
    } DaqCtrlStruct;

int     CVIFUNC InitDaqCtrl(DaqCtrl daqCtrl);
int     CVIFUNC GetDAQCtrlInfo(int panel, int daqCtrlId, char *type, DaqCtrl *daqCtrl);
int     CVIFUNC SetCtrlModeToIODirection(int panel, int control, int ioDirection);

int     CVIFUNC CreateDaqDlg(DaqCtrl daqCtrl);
int     CVIFUNC CreateDlgLists(DaqCtrl daqCtrl);
int     CVIFUNC CreateDlgSheetGroup(DaqCtrl daqCtrl);
int     CVIFUNC CreateErrorControls(DaqCtrl daqCtrl);
int     CVIFUNC CreateConfigCtrls(DaqCtrl daqCtrl);
int     CVIFUNC CreateGeneralCtrls(DaqCtrl daqCtrl);
int     CVIFUNC CreateDAQCtrls(DaqCtrl daqCtrl);
int     CVIFUNC CreateStdButtons(DaqCtrl daqCtrl);
int     CVIFUNC GetNumChannelsNoBOLE (int device, char *channel, int ioDirection, unsigned long *numChannels);
int     CVIFUNC FindDefaultDevice(int channelType, int *found);

int     CVICALLBACK DAQ_CtrlDialogCallback(int dialog, int control, int event, void *callbackData, int eventData1, int eventData2);
int     CVICALLBACK DAQ_CtrlDialogCallbackNoChange(int dialog, int control, int event, void *callbackData, int eventData1, int eventData2);

int     CVIFUNC DaqCtrl_RememberAttributes(DaqCtrl daqCtrl);
int     CVIFUNC DaqCtrl_RevertAttributes(DaqCtrl daqCtrl);
int     CVIFUNC DaqCtrl_CopyAttributesToDialog(DaqCtrl daqCtrl);
int     CVIFUNC DaqCtrl_ApplyDialogSettings(DaqCtrl daqCtrl);
int     CVIFUNC DaqCtrl_UpdateConfigDlgCtrl(DaqCtrl daqCtrl);
void    CVIFUNC DaqCtrl_DisposeDaqCtrl(DaqCtrl daqCtrl);
void    CVIFUNC DaqCtrl_DisplayError(DaqCtrl daqCtrl, int errorCode, double timeTillReplaceable, double timeTillClearable);
void    CVIFUNC DaqCtrl_ClearDisplayedError(DaqCtrl daqCtrl);
void    CVIFUNC DaqCtrl_SetErrorElaboration(DaqCtrl daqCtrl, char *elaboration);
int     CVIFUNC DaqCtrl_EditPopup(DaqCtrl daqCtrl, char *dialogTitle, int modal);
int     CVIFUNC DaqCtrl_CalcMinDialogWidth(DaqCtrl daqCtrl);
int     CVIFUNC DaqCtrl_FormatDialog(DaqCtrl daqCtrl);
int     CVIFUNC DaqCtrl_FinishDialogSizing(DaqCtrl daqCtrl);
int     CVIFUNC DaqCtrl_ReadSettings(DaqCtrl daqCtrl, IniText iniText, char *section, char *errorDescription, int *doingGets);
int     CVIFUNC DaqCtrl_WriteSettings(DaqCtrl daqCtrl, IniText iniText, char *section);
int     CVIFUNC DaqCtrl_UpdateDialogDimming(DaqCtrl daqCtrl);
void    CVIFUNC DaqCtrl_ChannelStringIsRevertable(DaqCtrl daqCtrl, int isRevertable);
int     CVIFUNC DaqCtrl_SetChangedState(DaqCtrl daqCtrl, int changed);
int     CVIFUNC DaqCtrl_SetAttributeFromParmInfo(DaqCtrl daqCtrl, int attribute, va_list parmInfo);
int     CVIFUNC DaqCtrl_GetAttribute(DaqCtrl daqCtrl, int attribute, void *value);
int     CVIFUNC DaqCtrl_RunRightClickPopupMenu(int panel, Point p);

#ifdef __cplusplus
    }
#endif

#endif /* DAQ_CTRL_HEADER */
