/*============================================================================*/
/*                        L a b W i n d o w s / C V I                         */
/*----------------------------------------------------------------------------*/
/*    Copyright (c) National Instruments 1987-1996.  All Rights Reserved.     */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/* Title:       datalog.h                                                     */
/* Purpose:     provides datalogging functions                                */
/*                                                                            */
/*============================================================================*/

#ifndef DATA_LOG_HEADER
#define DATA_LOG_HEADER

#include "inifile.h"

#ifdef __cplusplus
    extern "C" {
#endif


#define ATTR_DATALOG_LOGGING_ON                     3400
#define ATTR_DATALOG_LOGGING_PAUSED                 3401
#define ATTR_DATALOG_LOGGING_STATE                  3402
#define ATTR_DATALOG_DATA_LOG_FILE                  3403
#define ATTR_DATALOG_DATA_LOG_FILE_LENGTH           3404
#define ATTR_DATALOG_DATA_LOG_FILE_COPY             3405
#define ATTR_DATALOG_EVENT_LOG_FILE                 3406
#define ATTR_DATALOG_LOG_EVENTS                     3407
#define ATTR_DATALOG_LOG_FILE_FORMAT                3408
#define ATTR_DATALOG_START_LOGGING_ON_LOAD          3409   /* the value is only stored in the datalog object, the user of the object must implement the automatic log starting */
#define ATTR_DATALOG_PROMPT_ON_LOG_OVERWRITE        3410
#define ATTR_DATALOG_LOG_FIELD_WIDTH                3411
#define ATTR_DATALOG_WRITE_LOG_HEADER               3412
#define ATTR_DATALOG_COLUMN_HEADINGS                3413
#define ATTR_DATALOG_COLUMN_HEADINGS_COPY           3414
#define ATTR_DATALOG_COLUMN_HEADINGS_LENGTH         3415
#define ATTR_DATALOG_COMMENT                        3416
#define ATTR_DATALOG_COMMENT_COPY                   3417
#define ATTR_DATALOG_COMMENT_LENGTH                 3418
#define ATTR_DATALOG_LOG_MODE                       3419
#define ATTR_DATALOG_NUM_SCANS_TO_LOG               3420
#define ATTR_DATALOG_NUM_SCANS_LOGGED               3421
#define ATTR_DATALOG_EXTRA_HEADER_INFO              3422
#define ATTR_DATALOG_EXTRA_HEADER_INFO_COPY         3423
#define ATTR_DATALOG_EXTRA_HEADER_INFO_LENGTH       3424
#define ATTR_DATALOG_LOG_CONDITIONALLY              3425    /* the value is only stored in the datalog object, the user of the object must implement the conditional logging using ATTR_DATALOG_PAUSE_FOR_CONDITION */
#define ATTR_DATALOG_NUM_CHANNELS                   3426    /* must be set before each use, does not get written out */
#define ATTR_DATALOG_PAUSE_FOR_CONDITION            3427

#define DATALOG_MIN_USER_SETTABLE_FIELD_WIDTH       10
#define DATALOG_MAX_FIELD_WIDTH                     50

typedef enum
    {
    DATA_LOG_ACTIVE,
    DATA_LOG_PAUSED,
    DATA_LOG_STOPPED,
    DATA_LOG_WAITING_FOR_CONDITION,
    } DataLogState;

typedef enum
    {
    DATA_LOG_DONT_STOP,
    DATA_LOG_OVERWRITE_OLDEST,
    DATA_LOG_STOP_AT_SCAN_LIMIT,
    } DataLogMode;

typedef enum
    {
    DATALOG_CHANGED,                /* a datalog attribute has changed */
    DATALOG_CHANGE_REJECTED,        /* an attempt to change a datalog attribute had to be rejected */
    DATALOG_ERROR_ELABORATION,      /* eventData is pointer to error elaboration string */
    DATALOG_REVISE_COLUMN_HEADERS,  /* sent before writing column headers, eventData is a pointer to a string which can be freed and replaced with a substitute (which the datalog will dispose) */
    } DataLogNotification;

typedef enum
    {
    DATA_LOG_TAB_DELIMITED_TEXT,
    DATA_LOG_COMMA_DELIMITED_TEXT,
    DATA_LOG_CONSTANT_FIELD_WIDTH_TEXT,
    DATA_LOG_BINARY,
    } DataLogFormat;

#define DATALOG_NUM_BINARY_FIXED_HEADER_BYTES   (4 + sizeof(int) * 5)
    /*  data log binary file format:

        char    signature[4];       //  contains "DLOG"         
        int     numHeaderBytes;
        int     version;
        int     numChannels;
        int     writeHeader;        // boolean
        int     preferredFieldWidth;
            // variable amount of header data, skip using value of numHeaderBytes:
            char    timeAndDataString[];    // null terminated string
            char    columnHeadings[];       // null terminated string
            char    comment[];              // null terminated string
            ...                             // available for future expansion
        double  timeOfScan;
        double  scan[numChannels];
                //...               repeat time and scan until no more scans ...
    */



typedef struct DataLogStructTag *DataLog;       /* the abstract data type */
typedef void (CVICALLBACK * DataLogNotifyFunc)(void *callbackData, DataLogNotification event, void *eventData);

/* public data log functions */

DataLog CVIFUNC DataLog_Create(DataLogNotifyFunc notifyFunc, void *callbackData);
void    CVIFUNC DataLog_Dispose(DataLog dataLog);
int     CVIFUNC DataLog_SetAttributeFromParmInfo(DataLog dataLog, int attribute, va_list parmInfo);
int     CVIFUNC_C DataLog_SetAttribute(DataLog dataLog, int attribute, ...);
int     CVIFUNC DataLog_GetAttribute(DataLog dataLog, int attribute, void *value);
int     CVIFUNC DataLog_IsAttribute(int attribute);
int     CVIFUNC DataLog_TextFormatDlg(DataLog dataLog);
int     CVIFUNC ReadInDataLogAttributes(DataLog dataLog, IniText iniText, char *section, char *errorDescription, int *doingGets);
int     CVIFUNC WriteOutDataLogAttributes(DataLog dataLog, IniText iniText, char *section);
int     CVIFUNC DataLog_LogData(DataLog dataLog, double *time, void *data, int numRows);
int     CVIFUNC DataLog_LogMessage(DataLog dataLog, char message[], double *timeOfScan);
int     CVIFUNC DataLog_SelectLogFile(DataLog dataLog, int writingFileNow);


    /* return TRUE to cancel conversion? */

int     CVIFUNC DataLog_ConvertBinaryFileDlg(DataLog dataLog, int *fileWasConverted);
int     CVIFUNC DataLog_ReorderLogFile(char *logFilePath, int displayProgressDialog);




#ifdef __cplusplus
    }
#endif

#endif /* DATA_LOG_HEADER */
